from os import listdir
from os.path import isfile, join
from song_types import Audio
import re
from datetime import datetime


AUDIO_DIR = "audios/"

def find_audios(index):
    """
    Finds all audios in a folder that match the given index.
    Audios must be in the format [index]_[YYYY]-[MM]-[DD].mp3
    :param index: An integer denoting the song's index
    :return: A list of matching Audio objects
    """
    res = []
    for f in listdir(AUDIO_DIR):
        full_file = join(AUDIO_DIR, f)
        re_date_match = re.match(r"^%03d_(\d{4}-[01]\d-[0-3]\d).mp3$" % index, f)
        if not isfile(full_file) or not re_date_match:
            continue
        date = datetime.strptime(re_date_match.group(1), "%Y-%m-%d")
        res.append(Audio(date, join("../", full_file)))
    return res
